/*
 * This program was written by Grzegorz Adam Hankiewicz and Carlos Pereda
 * Moreno and is Giftware: you are free to do what you want with it without
 * any restriction. We do not accept responsibility for any effects, adverse
 * or otherwise, that this program may have on you, your computer, your
 * sanity, your dog, and anything else that you can think of. Use it at your
 * own risk.
*/
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.StreamTokenizer;

/** Esta clase es la que lee ficheros de mapas con informacin logica de
  * los coches, para transformar esta informacin en un array de casillas
  * y genera los coches en su lugar adecado. Implementa la lectura
  * usando un StreamTokenizer, ya que los ficheros de mapa son ficheros
  * ascii normales que pueden ser editados manualmente con cualquier
  * editor de texto. Los mapas son cargados dinmicamente por el juego,
  * as que no hace falta ni cerrar el programa para releer un nuevo mapa.
  */
public class MapaTexto
{
	public final static int DIR_ARRIBA = 1;
	public final static int DIR_ABAJO = 2;
	public final static int DIR_IZQUIERDA = 3;
	public final static int DIR_DERECHA = 4;

	public static void Lee (String nombre, Casilla mapa[][], int xmax, int ymax, Coche coches[], int cmax)
  		throws java.io.FileNotFoundException, java.io.IOException
	{
		// Abrimos el fichero
		FileReader file = new FileReader (nombre);
		// Lo tokenizamos
		StreamTokenizer st = new StreamTokenizer (new BufferedReader (file));
		// Especificamos que slo queremos nmeros y lneas
		st.resetSyntax();
		st.wordChars (32, 255);

		System.out.println ("Cargando mapa '" + nombre + "'");

		// Inicializacin de las casillas del mapa
		for (int y = 0; y < ymax; y++)
			for (int x = 0; x < xmax; x++)
				mapa[x][y] = new Casilla();

		// Empezamos el bucle leyendo nmeros
		int y = 0;
		while (StreamTokenizer.TT_EOF != st.nextToken() && y < ymax)
		{
			if (st.ttype == StreamTokenizer.TT_WORD)
			{
			 	for (int x = 0; x < xmax && x < st.sval.length(); x++)
					if (st.sval.charAt (x) != ' ')
						mapa[x][y].set_logico (1);
				y++;
			}
		}

		// Bucle para poner las convertir tiles
		for (y = 0; y < ymax; y++)
		{
			for (int x = 0; x < xmax; x++)
			{
				int up = 0, down = 0, left = 0, right = 0;

				if (y > 0) 			up = mapa[x][y-1].get_logico();
				if (y < ymax - 1) down = mapa[x][y+1].get_logico();
				if (x > 0) 			left = mapa[x-1][y].get_logico();
				if (x < xmax - 1) right = mapa[x+1][y].get_logico();

				mapa[x][y].set_tile_suelo (up, down, left, right, mapa[x][y].get_logico());
			}
		}

		// Vale, se supone que tenemos el mapa ledo... leemos coches.
		LeeCoches (st, coches, cmax);

		// Post-iniciacin de los coches para que se pinten
		for (int x = 0; x < cmax; x++)
			if (coches[x] != null)
				coches[x].autoinicia (mapa);
	}

	static void LeeCoches (StreamTokenizer st, Coche coches[], int cmax)
		throws java.io.IOException
	{
		int estado = 0, n_coches = 0;
		int x = 0, y = 0, dir = 0;

		// Las lneas son del tipo  Coche = posx, posy, direccion
		// Y los estados son:	0	   1      2      3       4
		// Para pasar a un estado hay que estar en el siguiente. Si se salta
		// alguno, se aborta la cadena y se vuelve al estado cero, excepto
		// cuando se leen palabras.

		// Activamos las letras como "palabras", sin maysculas e ignoramos
		// algunos signos de puntuacin
		st.resetSyntax();
		st.parseNumbers();
		st.eolIsSignificant (true);
		st.wordChars (65, 122);
		st.lowerCaseMode (true);
		st.whitespaceChars (44, 45);
		st.whitespaceChars (58, 63);

		// Leemos palabras y eso...
		while (StreamTokenizer.TT_EOF != st.nextToken() && n_coches < cmax)
		{
			// Detectamos nmero y fin de lnea
			switch (st.ttype)
			{
				case StreamTokenizer.TT_NUMBER:
					if (estado == 1) { x = (int) st.nval; estado++; }
					else if (estado == 2) { y = (int) st.nval; estado++; }
					else estado = 0;
					break;
					
				case StreamTokenizer.TT_EOL:
					estado = 0;
					break;

            case StreamTokenizer.TT_WORD:
					// Ignoramos palabras que no sean coche o direcciones
					if (st.sval.compareTo ("coche") == 0)
						estado = 1;
					else if (estado == 3)
					{
						if (st.sval.compareTo ("arriba") == 0) {estado++; dir = DIR_ARRIBA;}
						else if (st.sval.compareTo ("abajo") == 0) {estado++; dir = DIR_ABAJO;}
						else if (st.sval.compareTo ("izquierda") == 0) {estado++; dir = DIR_IZQUIERDA;}
						else if (st.sval.compareTo ("derecha") == 0) {estado++; dir = DIR_DERECHA;}
					}
               break;

				default:
					break;
			}

			// Comprobamos si hemos llegado al estado 4
			if (estado == 4)
			{	// Aadimos el coche.
				coches[n_coches] = new Coche (x, y, dir, n_coches);
				estado = 0;
				n_coches++;
			}
		}

		if (n_coches != cmax)
		{
			System.out.println ("No se leyeron suficientes coches");
			System.exit (1);
		}
	}
}

